/**
 * @import {Element, Text} from 'hast'
 * @import {InlineCode} from 'mdast'
 * @import {State} from '../state.js'
 */
/**
 * Turn an mdast `inlineCode` node into hast.
 *
 * @param {State} state
 *   Info passed around.
 * @param {InlineCode} node
 *   mdast node.
 * @returns {Element}
 *   hast node.
 */
export function inlineCode(state: State, node: InlineCode): Element;
import type { State } from '../state.js';
import type { InlineCode } from 'mdast';
import type { Element } from 'hast';
//# sourceMappingURL=inline-code.d.ts.map