/**
 * Add support GFM (autolink literals, footnotes, strikethrough, tables,
 * tasklists).
 *
 * @param {Options | null | undefined} [options]
 *   Configuration (optional).
 * @returns {undefined}
 *   Nothing.
 */
export default function remarkGfm(options?: Options | null | undefined): undefined;
import type { Options } from 'remark-gfm';
//# sourceMappingURL=index.d.ts.map