/**
 * @import {Parents as HastParents, Root as HastRoot} from 'hast'
 * @import {Root as MdastRoot} from 'mdast'
 * @import {State} from '../state.js'
 */
/**
 * Turn an mdast `root` node into hast.
 *
 * @param {State} state
 *   Info passed around.
 * @param {MdastRoot} node
 *   mdast node.
 * @returns {HastParents}
 *   hast node.
 */
export function root(state: State, node: MdastRoot): HastParents;
import type { State } from '../state.js';
import type { Root as MdastRoot } from 'mdast';
import type { Parents as HastParents } from 'hast';
//# sourceMappingURL=root.d.ts.map