/**
 * Turn an mdast `footnoteReference` node into hast.
 *
 * @param {State} state
 *   Info passed around.
 * @param {FootnoteReference} node
 *   mdast node.
 * @returns {Element}
 *   hast node.
 */
export function footnoteReference(state: State, node: FootnoteReference): Element;
import type { State } from '../state.js';
import type { FootnoteReference } from 'mdast';
import type { Element } from 'hast';
//# sourceMappingURL=footnote-reference.d.ts.map