/**
 * @param {ReadonlyArray<Schema>} definitions
 *   Definitions.
 * @param {Space | undefined} [space]
 *   Space.
 * @returns {Schema}
 *   Schema.
 */
export function merge(definitions: ReadonlyArray<Schema>, space?: Space | undefined): Schema;
import { Schema } from './schema.js';
import type { Space } from 'property-information';
//# sourceMappingURL=merge.d.ts.map