/**
 * Turn an mdast `text` node into hast.
 *
 * @param {State} state
 *   Info passed around.
 * @param {MdastText} node
 *   mdast node.
 * @returns {HastElement | HastText}
 *   hast node.
 */
export function text(state: State, node: MdastText): HastElement | HastText;
import type { State } from '../state.js';
import type { Text as MdastText } from 'mdast';
import type { Element as HastElement } from 'hast';
import type { Text as HastText } from 'hast';
//# sourceMappingURL=text.d.ts.map