import type { StateCreator, StoreMutatorIdentifier } from 'zustand/vanilla';
export declare function ssrSafe<T extends object, U extends object, Mps extends [StoreMutatorIdentifier, unknown][] = [], Mcs extends [StoreMutatorIdentifier, unknown][] = []>(config: StateCreator<T, Mps, Mcs, U>, isSSR?: boolean): StateCreator<T, Mps, Mcs, U>;
