module.exports={A:{A:{"2":"K D E F A B zC"},B:{"1":"0 1 2 3 4 5 6 7 8 w x y z JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB I","2":"C L M G N O P Q","578":"H R S T U V W X Y Z a b c","1602":"d e f g h i j k l m n o p q r s t u v"},C:{"2":"9 0C VC J bB K D E F A B C L M G N O P cB AB BB CB DB EB FB GB HB IB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B WC 6B XC 7B 4C 5C","194":"0 1 2 3 4 5 6 7 8 8B 9B AC BC CC DC EC FC GC HC IC JC KC LC MC NC Q H R YC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB","4292":"YB ZB aB I","16580":"ZC aC OC 1C 2C 3C"},D:{"2":"9 J bB K D E F A B C L M G N O P cB AB BB CB DB EB FB GB HB IB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B WC 6B XC 7B 8B 9B AC BC CC DC EC FC GC HC IC JC KC LC MC NC Q","578":"H R S T U V W X Y Z a b c","1602":"d e f g h i j k l m n o p q r s t u v","2049":"0 1 2 3 4 5 6 7 8 w x y z JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB I ZC aC OC"},E:{"2":"J bB K D E F A B G 6C bC 7C 8C 9C AD cC DD dC eC RC ED SC fC gC hC iC jC FD TC kC lC mC","322":"C L M PC QC BD CD nC oC GD UC pC qC rC sC HD","8452":"tC uC vC wC ID"},F:{"2":"9 F B C G N O P cB AB BB CB DB EB FB GB HB IB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC GC HC JD KD LD MD PC xC ND QC","578":"IC JC KC LC MC NC Q H R YC S T U V W X Y Z a b c d e f g h","2049":"0 1 2 3 4 5 6 7 8 i j k l m n o p q r s t u v w x y z"},G:{"1":"tC uC vC wC","2":"E bC OD yC PD QD RD SD TD UD VD WD XD YD ZD aD bD cD dD eD fD gD hD dC eC RC iD SC fC gC hC iC jC jD TC kC lC mC","322":"nC oC kD UC pC qC rC sC lD"},H:{"2":"mD"},I:{"2":"VC J I nD oD pD qD yC rD sD"},J:{"2":"D A"},K:{"2":"A B C PC xC QC","2049":"H"},L:{"1":"I"},M:{"194":"OC"},N:{"2":"A B"},O:{"2":"RC"},P:{"1":"DB EB FB GB HB IB","2":"9 J AB BB CB tD uD vD wD xD cC yD zD 0D 1D 2D SC TC UC 3D"},Q:{"2":"4D"},R:{"2":"5D"},S:{"2":"6D","194":"7D"}},B:5,C:"WebGPU",D:true};
