/**
 * @import {Element, Text} from 'hast'
 * @import {Break} from 'mdast'
 * @import {State} from '../state.js'
 */
/**
 * Turn an mdast `break` node into hast.
 *
 * @param {State} state
 *   Info passed around.
 * @param {Break} node
 *   mdast node.
 * @returns {Array<Element | Text>}
 *   hast element content.
 */
export function hardBreak(state: State, node: Break): Array<Element | Text>;
import type { State } from '../state.js';
import type { Break } from 'mdast';
import type { Element } from 'hast';
import type { Text } from 'hast';
//# sourceMappingURL=break.d.ts.map