/**
 * @param {Heading} node
 * @param {State} state
 * @returns {boolean}
 */
export function formatHeadingAsSetext(node: Heading, state: State): boolean;
import type { Heading } from 'mdast';
import type { State } from 'mdast-util-to-markdown';
//# sourceMappingURL=format-heading-as-setext.d.ts.map