module.exports={A:{A:{"2":"K D E F A B zC"},B:{"1":"0 1 2 3 4 5 6 7 8 W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB I","2":"C L M G N O P","322":"Q H R S T","578":"U V"},C:{"1":"0 1 2 3 4 5 6 7 8 X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB I ZC aC OC 1C 2C 3C","2":"9 0C VC J bB K D E F A B C L M G N O P cB AB BB CB DB EB FB GB HB IB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B WC 6B XC 7B 8B 9B AC BC CC DC EC FC GC 4C 5C","194":"HC IC JC KC LC MC NC Q H R YC S T U V W"},D:{"1":"0 1 2 3 4 5 6 7 8 W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z JB KB LB MB NB OB PB QB RB SB TB UB VB WB XB YB ZB aB I ZC aC OC","2":"9 J bB K D E F A B C L M G N O P cB AB BB CB DB EB FB GB HB IB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B WC 6B XC 7B 8B 9B AC BC CC DC EC FC GC HC IC JC KC LC MC NC","322":"Q H R S T","578":"U V"},E:{"1":"UC pC qC rC sC HD tC uC vC wC ID","2":"J bB K D E F A B C L 6C bC 7C 8C 9C AD cC PC QC BD","2049":"iC jC FD TC kC lC mC nC oC GD","2113":"SC fC gC hC","3140":"M G CD DD dC eC RC ED"},F:{"1":"0 1 2 3 4 5 6 7 8 JC KC LC MC NC Q H R YC S T U V W X Y Z a b c d e f g h i j k l m n o p q r s t u v w x y z","2":"9 F B C G N O P cB AB BB CB DB EB FB GB HB IB dB eB fB gB hB iB jB kB lB mB nB oB pB qB rB sB tB uB vB wB xB yB zB 0B 1B 2B 3B 4B 5B 6B 7B 8B 9B AC BC CC DC EC FC GC HC JD KD LD MD PC xC ND QC","578":"IC"},G:{"1":"UC pC qC rC sC lD tC uC vC wC","2":"E bC OD yC PD QD RD SD TD UD VD WD XD YD ZD aD bD cD dD eD","2049":"iC jC jD TC kC lC mC nC oC kD","2113":"SC fC gC hC","2116":"fD gD hD dC eC RC iD"},H:{"2":"mD"},I:{"1":"I","2":"VC J nD oD pD qD yC rD sD"},J:{"2":"D A"},K:{"1":"H","2":"A B C PC xC QC"},L:{"1":"I"},M:{"1":"OC"},N:{"2":"A B"},O:{"1":"RC"},P:{"1":"vD","2":"9 J AB BB CB DB EB FB GB tD uD wD xD cC yD zD 0D 1D 2D SC TC UC 3D","4098":"HB IB"},Q:{"2":"4D"},R:{"1":"5D"},S:{"2":"6D 7D"}},B:6,C:"HTTP/3 protocol",D:true};
