export const boolean: number;
export const booleanish: number;
export const overloadedBoolean: number;
export const number: number;
export const spaceSeparated: number;
export const commaSeparated: number;
export const commaOrSpaceSeparated: number;
//# sourceMappingURL=types.d.ts.map