"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "tap", {
    enumerable: true,
    get: function() {
        return tap;
    }
});
function tap(value, mutator) {
    mutator(value);
    return value;
}
