/**
 * @param {URL | string} path
 *   File URL.
 * @returns {string}
 *   File URL.
 */
export function urlToPath(path: URL | string): string;
export { isUrl } from "./minurl.shared.js";
//# sourceMappingURL=minurl.browser.d.ts.map