/**
 * @param {Link} node
 * @param {State} state
 * @returns {boolean}
 */
export function formatLinkAsAutolink(node: Link, state: State): boolean;
import type { Link } from 'mdast';
import type { State } from 'mdast-util-to-markdown';
//# sourceMappingURL=format-link-as-autolink.d.ts.map