/**
 * Turn an mdast `linkReference` node into hast.
 *
 * @param {State} state
 *   Info passed around.
 * @param {LinkReference} node
 *   mdast node.
 * @returns {Array<ElementContent> | ElementContent}
 *   hast node.
 */
export function linkReference(state: State, node: LinkReference): Array<ElementContent> | ElementContent;
import type { State } from '../state.js';
import type { LinkReference } from 'mdast';
import type { ElementContent } from 'hast';
//# sourceMappingURL=link-reference.d.ts.map