/**
 * Turn an mdast `link` node into hast.
 *
 * @param {State} state
 *   Info passed around.
 * @param {Link} node
 *   mdast node.
 * @returns {Element}
 *   hast node.
 */
export function link(state: State, node: Link): Element;
import type { State } from '../state.js';
import type { Link } from 'mdast';
import type { Element } from 'hast';
//# sourceMappingURL=link.d.ts.map