export function lazyPostcss() {
  return require('postcss')
}

export function lazyPostcssImport() {
  return require('postcss-import')
}

export function lazyAutoprefixer() {
  return require('autoprefixer')
}

export function lazyCssnano() {
  return require('cssnano')
}
