/**
 * @param {Record<string, string>} attributes
 *   Attributes.
 * @param {string} property
 *   Property.
 * @returns {string}
 *   Transformed property.
 */
export function caseInsensitiveTransform(attributes: Record<string, string>, property: string): string;
//# sourceMappingURL=case-insensitive-transform.d.ts.map